{#############################################################################}
{#                                                                           #}
{#  WinQUANT v2 - source code                                                #}
{#  Delphi version                                                           #}
{#  File: BookmarkManager.pas v2.0                                           #}
{#                                                                           #}
{#  (C) 2001 Petr Strnsk                                                   #}
{#  All rights reserved                                                      #}
{#                                                                           #}
{#  Web:     http://www.fi.muni.cz/~xstransk/winquant                        #}
{#  E-mail:  xstransk@fi.muni.cz                                             #}
{#           aargh@email.cz                                                  #}
{#                                                                           #}
{#############################################################################}
{#                                                                           #}
{#  This file is distributed as freeware and without warranties of any kind. #}
{#  You can use it in your own applications at your own risk.                #}
{#  See help file for more information.                                      #}
{#                                                                           #}
{#############################################################################}

unit BookmarkManager;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Dialogs, Controls,
  StdCtrls, Buttons, Bookmark;

type
  TBookmarkManagerDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    BookmarkList: TListBox;
    SrcLabel: TLabel;
    RenameBtn: TButton;
    DeleteBtn: TButton;
    ImportBtn: TButton;
    CopyBtn: TButton;
    UpBitBtn: TBitBtn;
    DownBitBtn: TBitBtn;
    SortBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    Label1: TLabel;
    ImportList: TListBox;
    IncludeBtn: TButton;
    IncAllBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ImportBtnClick(Sender: TObject);         {select import file}
    procedure IncludeBtnClick(Sender: TObject);        {copy selected bookmark from import file}
    procedure IncAllBtnClick(Sender: TObject);         {copy all bookmarks from import file}
    procedure ArrowBtnClick(Sender: TObject);          {change position of a bookmark}
    procedure DeleteBtnClick(Sender: TObject);         {delete bookmark}
    procedure CopyRenameBtnClick(Sender: TObject);     {copy or rename bookmark - it handles both Copy and Rename button}
    procedure SortBtnClick(Sender: TObject);           {sort bookmarks alphabetically by name}
    procedure ListClick(Sender: TObject);              {select bookmark and set its name as a hint}
  private
    procedure UpdateBookmarkList(ItemIndex: integer = -1);   {update the list of bookmarks}
  public
    function Execute(Bookmarks: TBookmarks): TModalResult;   {execute the bookmark manager - do NOT use Show or ShowModal directly}
    function GetBookmarks: TBookmarks;                 {returns changed version of bookmarks}
  end;

var
  BookmarkManagerDlg: TBookmarkManagerDlg;

implementation

{$R *.DFM}

uses Math {because of Min, Max};

var TmpBookmarks: TBookmarks;
    ImportBookmarks: TBookmarks;

procedure TBookmarkManagerDlg.FormCreate(Sender: TObject);
begin
  TmpBookmarks := TBookmarks.Create;
  ImportBookmarks := TBookmarks.Create;
end;

procedure TBookmarkManagerDlg.FormDestroy(Sender: TObject);
begin
  TmpBookmarks.Destroy;
  ImportBookmarks.Destroy;
end;

function TBookmarkManagerDlg.Execute(Bookmarks: TBookmarks): TModalResult;
begin
  TmpBookmarks.Copy(Bookmarks);
  UpdateBookmarkList;
  Result := ShowModal;
end;

function TBookmarkManagerDlg.GetBookmarks: TBookmarks;
begin
  Result := TmpBookmarks;
end;

procedure TBookmarkManagerDlg.UpdateBookmarkList(ItemIndex: integer);
var loop: integer;
begin
  BookmarkList.Clear;
  for loop := 0 to TmpBookmarks.Count-1 do
    BookmarkList.Items[loop] := TmpBookmarks.Getname(loop);
  BookmarkList.ItemIndex := ItemIndex;
end;

procedure TBookmarkManagerDlg.ImportBtnClick(Sender: TObject);
var loop: integer;
begin
  with OpenDialog do
    if Execute then begin
      ImportBookmarks.FileName := FileName;
      ImportBookmarks.LoadFromFile;
    end;
  for loop := 0 to ImportBookmarks.Count-1 do
    ImportList.Items[loop] := ImportBookmarks.Getname(loop);
end;

procedure TBookmarkManagerDlg.IncludeBtnClick(Sender: TObject);
var loop: integer;
begin
  for loop := 0 to ImportList.Items.Count-1 do
    if ImportList.Selected[loop] then
      TmpBookmarks.AddMark(ImportBookmarks.GetMark(loop));
  UpdateBookmarkList;
end;

procedure TBookmarkManagerDlg.IncAllBtnClick(Sender: TObject);
var loop: integer;
begin
  for loop := 0 to ImportBookmarks.Count-1 do
    TmpBookmarks.AddMark(ImportBookmarks.GetMark(loop));
  UpdateBookmarkList;
end;

procedure TBookmarkManagerDlg.ArrowBtnClick(Sender: TObject);
var Index: integer;
begin
  Index := BookmarkList.ItemIndex;
  if Sender = UpBitBtn then begin
    TmpBookmarks.MoveMark(Index, Index-1);
    UpdateBookmarkList(Max(0, Index-1));
  end else begin
    TmpBookmarks.MoveMark(Index, Index+1);
    UpdateBookmarkList(Min(BookmarkList.Items.Count-1, Index+1));
  end;
end;

procedure TBookmarkManagerDlg.DeleteBtnClick(Sender: TObject);
begin
  TmpBookmarks.DeleteMark(BookmarkList.ItemIndex);
  UpdateBookmarkList;
end;

procedure TBookmarkManagerDlg.CopyRenameBtnClick(Sender: TObject);
var NameStr: string;
begin
  try
    with BookmarkList do NameStr := Items[ItemIndex];
    if InputQuery('Bookmark name', 'Type new bookmark''s name', NameStr) then
      with TmpBookmarks do
        if Sender = CopyBtn then AddMark(GetMark(BookmarkList.ItemIndex), NameStr)
        else RenameMark(BookmarkList.ItemIndex, NameStr);
    UpdateBookmarkList;
  except
    {ignore all exceptions}
  end;
end;

procedure TBookmarkManagerDlg.SortBtnClick(Sender: TObject);
begin
  TmpBookmarks.Sort;
  UpdateBookmarkList;
end;

procedure TBookmarkManagerDlg.ListClick(Sender: TObject);
begin
  if Sender = BookmarkList then
    BookmarkList.Hint := TmpBookmarks.GetName(BookmarkList.ItemIndex)
  else
    ImportList.Hint := ImportBookmarks.GetName(ImportList.ItemIndex);
end;

end.
