{#############################################################################}
{#                                                                           #}
{#  WinQUANT v2 - source code                                                #}
{#  Delphi version                                                           #}
{#  File: Calculations.pas v2.0                                              #}
{#                                                                           #}
{#  (C) 2001 Petr Strnsk                                                   #}
{#  All rights reserved                                                      #}
{#                                                                           #}
{#  Web:     http://www.fi.muni.cz/~xstransk/winquant                        #}
{#  E-mail:  xstransk@fi.muni.cz                                             #}
{#           aargh@email.cz                                                  #}
{#                                                                           #}
{#############################################################################}
{#                                                                           #}
{#  This file is distributed as freeware and without warranties of any kind. #}
{#  You can use it in your own applications at your own risk.                #}
{#  See help file for more information.                                      #}
{#                                                                           #}
{#############################################################################}

unit Calculations;

interface

uses SysUtils, Forms, Graphics, Classes, Document, UParser,
     Dialogs {because of ShowMessage};

type
  TProgressEvent = procedure(Sender: TObject; Progress: byte) of object;

  TPBitmap = ^TBitmap;

  TCalculator = class(TObject)
    public
      MinVal: single;                {minimum value in current image}
      MaxVal: single;                {maximum value in current image}
      StopIt: boolean;               {end and exit ?}
      constructor Create (AOwner: TComponent);
      function Calculate(PBitmap: TPBitmap; TmpDoc: TDocument):boolean;
                                     {draw document on bitmap}
    private
      Parser: TParser;               {analyzator and evaluator}
      FOnProgress : TProgressEvent;  {executed every fifth row}
    published
      property OnProgress : TProgressEvent read FOnProgress write FOnProgress;
                                     {OnProgress event handler stored here}
  end;

  ECalcError = class(Exception) end;

implementation

 {* FUNCTIONS *}

constructor TCalculator.Create;
begin
  Parser := TParser.Create;
end;

function TCalculator.Calculate(PBitmap: TPBitmap; TmpDoc: TDocument):boolean;
var I, J: integer;          {coordinates of current point}
    XPW, YPW: real;         {horizontal and vertical coordinate increase per pixel}
    Value: real;            {result value}
    XRes, YRes: integer;    {horizontal and vertical resolution}
    SwapTmp: single;
begin
  try
    Parser.ParseString := TmpDoc.Func;            {read function string}
    Parser.Prepare;                               {analyze function andtranslate to postfix}

    TmpDoc.GetRes(Xres, YRes);
    MaxVal := -3.4e38;
    MinVal := 3.4e38;
    StopIt := false;                              {init variables}

    PBitmap^.Width := XRes;
    PBitmap^.Height := YRes;                    {set bitmap resolution}

{      with PBitmap^.Canvas do begin
        Brush.Color := clBlack;
        FillRect(Rect(0,0,XRes,YRes));
      end;}

    with TmpDoc do begin
      XPW := (XT - XF) / XRes;
    	YPW := (YT - YF) / YRes;                  {horizontal and vertical coordinate increase per pixel}

      if RangeF = RangeT then
        raise ECalcError.Create('ERROR: Value range ''from'' and ''to'' values must not be equal');
      if RangeF > RangeT then begin             {RangeF must be lesser than RangeT}
        SwapTmp := RangeF;
        RangeF := RangeT;
        RangeT := SwapTmp;
      end;
    end;

    for J := 0 to YRes-1 do begin
      for I := 0 to XRes-1 do begin
        with Parser do begin
          XVar := TmpDoc.XF + I * XPW;         {display range coordinates of a currect point}
          YVar := TmpDoc.YF + J * YPW;

          try
            Evaluate;                          {execute the parser}
            if MathError then begin            {whet the result is undefined use NaN color}
              PBitmap^.Canvas.Pixels[i,j] := TmpDoc.UndefColor;
              Continue;
            end;
            Value := Result;
          except
            on EOverFlow do Value := TmpDoc.RangeT;
            on EUnderflow do Value := TmpDoc.RangeF;
          end;
        end;

        if MinVal > Value then MinVal := Value;
        if MaxVal < Value then MaxVal := Value;    {update minimal and maximal values}

        with TmpDoc do begin
          if Value > RangeT then Value := RangeT
          else if Value < RangeF then Value := RangeF;
          PBitmap^.Canvas.Pixels[i,j] :=
            GetColorGlobal((Value-RangeF)/(RangeT-RangeF), Point(I,J));  {find a color for the value}
        end;               {percentual position of Value in <RangeF, RangeT>}
      end;
      if (Assigned(OnProgress)) and (J mod 5 = 0) then
        OnProgress(Self, Trunc(j/YRes*100));      {execute event}
      Application.ProcessMessages;    {other process wants to do anything?}
      if StopIt then begin
        calculate := false;           {abort calculation?}
        Exit;
      end;
    end;
    calculate := true;
  except
    on Err: EParseError do begin      {handle syntax errors}
      MessageDlg(Err.Message, mtError, [mbOK], 0);
      calculate := false;
    end;
    on Err: ECalcError do begin
      MessageDlg(Err.Message, mtError, [mbOK], 0);
      calculate := false;             {handle RangeF=RangeT event}
    end;
  end;
end;

end.


