{#############################################################################}
{#                                                                           #}
{#  WinQUANT v2 - source code                                                #}
{#  Delphi version                                                           #}
{#  File: Result.pas v2.0                                                    #}
{#                                                                           #}
{#  (C) 2001 Petr Strnsk                                                   #}
{#  All rights reserved                                                      #}
{#                                                                           #}
{#  Web:     http://www.fi.muni.cz/~xstransk/winquant                        #}
{#  E-mail:  xstransk@fi.muni.cz                                             #}
{#           aargh@email.cz                                                  #}
{#                                                                           #}
{#############################################################################}
{#                                                                           #}
{#  This file is distributed as freeware and without warranties of any kind. #}
{#  You can use it in your own applications at your own risk.                #}
{#  See help file for more information.                                      #}
{#                                                                           #}
{#############################################################################}

unit Result;

interface

uses ExtCtrls, Dialogs, Menus, Controls, Forms, Classes,
     Graphics, Document, ComCtrls;
type
  TResultWin = class(TForm)
    ImageSaveDialog: TSaveDialog;
    DisplayScroll: TScrollBox;
    DisplayBox: TPaintBox;
    MainMenu: TMainMenu;
    SaveAbort: TMenuItem;
    ProgressBar: TProgressBar;
    StatusBar: TStatusBar;
    Preferences: TMenuItem;
  {* PROCEDURES *}
    procedure CenterPaintBox;                         {center image int the window}
    procedure CalcWinProgress(Sender: TObject; Progress: byte);  {update progress bar}
    procedure Execute(Doc: TDocument);                {execute result window and calculation}
    procedure Abort;                                  {abort calculation}
    procedure GetLastMinMax (out Min, Max: single);   {return minimum and maximum}
  {* GLOBAL EVENTS *}
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure DisplayBoxPaint(Sender: TObject);       {paint Bitmap}
  {* MENU EVENTS *}
    procedure ImageSave;                              {save image}
    procedure SaveAbortClick(Sender: TObject);        {save/abort}
    procedure PreferencesClick(Sender: TObject);      {show preferences}
  end;

var ResultWin: TResultWin;

implementation

uses SysUtils, Calculations, JPEG, Preferences;

var Bitmap: TBitmap;                   {current image}
    TmpDoc: TDocument;                 {temporary document}
    Calculator: TCalculator;           {calculator device}
    JPEGQuality: byte;                 {curent JPWG quality}

{$R *.DFM}

 {* PROCEDURES *}

 procedure TResultWin.CenterPaintBox;
var Tmp: integer;
begin
  Tmp := DisplayScroll.Width - DisplayBox.Width-3; {border size is 3?}
  if Tmp<=0 then DisplayBox.Left := 0
  else DisplayBox.Left := Tmp div 2;
  Tmp := DisplayScroll.Height - DisplayBox.Height-3;
  if Tmp<0 then DisplayBox.Top := 0
  else DisplayBox.Top := Tmp div 2;
end;

procedure TResultWin.CalcWinProgress(Sender: TObject; Progress: byte);
begin
  ProgressBar.Position := Progress;
end;

procedure TResultWin.Execute(Doc: TDocument);
begin
  TmpDoc.Copy(Doc);                             {temporary copy of document, because 'doc' itself could be changed while working}
  Calculator.OnProgress := CalcWinProgress;     {set 'progress' event handler}
  SaveAbort.Tag := 1;
  SaveAbort.Caption := 'Abort';
  Statusbar.Visible := false;
  ProgressBar.Visible := true;
  DisplayBox.Width := Doc.GetXRes;
  DisplayBox.Height := Doc.GetYRes;
  CenterPaintBox;
  WindowState := wsNormal;
  Show;
  DisplayBox.RePaint;
  Screen.Cursor := crAppStart;
  Calculator.Calculate(@Bitmap, TmpDoc);
  Screen.Cursor := crDefault;
  ProgressBar.Visible := false;
  StatusBar.Panels[0].Text := 'Minimal value = ' + FloatToStr(Calculator.MinVal);
  StatusBar.Panels[1].Text := 'Maximal value = ' + FloatToStr(Calculator.MaxVal);
  Statusbar.Visible := true;
  SaveAbort.Tag := 0;
  SaveAbort.Caption := 'Save image';
  FormResize(Self);
end;

procedure TResultWin.Abort;
begin
  Calculator.StopIt := true;
end;

procedure TResultWin.GetLastMinMax (out Min, Max: single);
begin
  Min := Calculator.MinVal;
  Max := Calculator.MaxVal;
end;

 {* GLOBAL EVENTS *}

procedure TResultWin.FormCreate(Sender: TObject);
begin
  TmpDoc := TDocument.Create;
  Calculator := TCalculator.Create(ResultWin);
  Bitmap := TBitmap.Create;             {inicializace bitmapy}
  Bitmap.PixelFormat := pf32bit;
  JPEGQuality := 100;
end;

procedure TResultWin.FormDestroy(Sender: TObject);
begin
  Abort;
  Bitmap.Destroy;
  Calculator.Destroy;
  TmpDoc.Destroy;
end;

procedure TResultWin.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caMinimize;
end;

procedure TResultWin.FormResize(Sender: TObject);
begin
  CenterPaintBox;
  DisplayBox.Invalidate;
  StatusBar.Panels[0].Width := StatusBar.Width div 2;
end;

procedure TResultWin.DisplayBoxPaint(Sender: TObject);
begin
  DisplayBox.Canvas.Draw (0,0,Bitmap);
end;

procedure TResultWin.ImageSave;
var JP: TJPEGImage;
begin
  with ImageSaveDialog do begin
    FileName := 'Image';
    if Execute then begin
      if FilterIndex=1 then begin   {save JPEG}
        JP := TJPEGImage.Create;
        try
          JP.Assign(Bitmap);
          JP.CompressionQuality := JPEGQuality;
          JP.SaveToFile(ImageSaveDialog.FileName)
        finally
          JP.Free;
        end;
      end else                      {save BMP}
        Bitmap.SaveToFile(ImageSaveDialog.FileName);
    end;
  end;
end;

procedure TResultWin.SaveAbortClick(Sender: TObject);
begin
  if SaveAbort.Tag = 0 then ImageSave
  else Calculator.StopIt := true;
end;

procedure TResultWin.PreferencesClick(Sender: TObject);
begin
  PreferencesDlg.QualityTrack.Position := JPEGQuality;
  if PreferencesDlg.ShowModal = mrOK then
    JPEGQuality := PreferencesDlg.QualityTrack.Position;
end;

end.
