{#############################################################################}
{#                                                                           #}
{#  WinQUANT v2 - source code                                                #}
{#  Delphi version                                                           #}
{#  File: UParserStack.pas v2.0                                              #}
{#                                                                           #}
{#  (C) 2001 Petr Strnsk                                                   #}
{#  All rights reserved                                                      #}
{#                                                                           #}
{#  Web:     http://www.fi.muni.cz/~xstransk/winquant                        #}
{#  E-mail:  xstransk@fi.muni.cz                                             #}
{#           aargh@email.cz                                                  #}
{#                                                                           #}
{#############################################################################}
{#                                                                           #}
{#  This file is distributed as freeware and without warranties of any kind. #}
{#  You can use it in your own applications at your own risk.                #}
{#  See help file for more information.                                      #}
{#                                                                           #}
{#############################################################################}

unit UParserStack;

interface

uses SysUtils;

type TItemType = ({SPECIAL}
                    itEOL {end of line}, itError,
                  {BRACKETS}
                    itLeft {left parenthesis}, itRight {right parenthesis},
                  {VALUES AND CONSTANTS}
                    itNum {number}, itXVar, itYVar {variables}, itPi,
                  {NULARY OPERATORS}
                    itRnd {nullary operator}, itRndSgn {1 or -1},
                  {UNARY OPERATORS}
                    itUnaryMinus,
                    itfAbs, itfATan, itfCos, itfExp, itfLn, itfRound, itfSin,
                    itfSqr, itfSqrt, itfTrunc,
                    itfACos, itfACosh, itfASin, itfASinh, itfATanh, itfCosh,
                    itfCotan, itfSinh, itfTan, itfTanh, itfFrac, itfSgn,
                  {BINARY OPERATORS}
                    itPlus, itMinus, itTimes, itDivide, itAng, itModulo, itExp,
                    itHypot, itLogN, itMax, itMin, itRandG);

     TStackValue = record
                     Value: single;
                     ItemType: TItemType;
                   end;

     TPStackUnit = ^TStackUnit;

     TStackUnit = record
       Value: TStackValue;
       Next: TPStackUnit;
       Prev: TPStackUnit;
     end;

{*** Optimized stack for UParser ***}

     TParserStack = class
       Top: TPStackUnit;
       constructor Create;
       destructor Destroy; override;
       procedure ClearStack;                    {clear stack}
       procedure Reverse;                       {reverse stacl}
       procedure Copy(Source: TParserStack);    {erase stack and copy from Source}
       procedure Push(Val: TStackValue);        {push value}
       function Pop: TStackValue;               {pop value}
       function Peek: TStackValue;              {what's on top?}
       function Empty: boolean;                 {is stack empty?}
     end;

     EStackEmpty = class(Exception)
     end;

implementation

constructor TParserStack.Create;
begin
  Top := nil;
end;

destructor TParserStack.Destroy;
begin
  ClearStack;
end;

procedure TParserStack.ClearStack;
begin
  while not Empty do Pop;
end;

procedure TParserStack.Reverse;
var It, Tmp: TPStackUnit;
begin
  It := Top;
  while It <> nil do begin
    Tmp := It^.Next;
    It^.Next := It^.Prev;
    It^.Prev := Tmp;
    It := Tmp;
    if It <> nil then Top := It;
  end;

end;

procedure TParserStack.Copy(Source: TParserStack);
var It: TPStackUnit;
begin
  ClearStack;
  It := Source.Top;
  if It <> nil then begin
    while It^.Next <> nil do It := It^.Next;
    while It <> nil do begin
      Push(It^.Value);
      It := It^.Prev;
    end;
  end;
end;

procedure TParserStack.Push(Val: TStackValue);
var NewUnit:TPStackUnit;
begin
  New(NewUnit);
  NewUnit^.Value := Val;
  NewUnit^.Next := Top;
  NewUnit^.Prev := nil;
  if Top <> nil then Top.Prev := NewUnit;
  Top := NewUnit;
end;

function TParserStack.Pop: TStackValue;
var DelUnit: TPStackUnit;
begin
  if Empty then raise EStackEmpty.Create('INTERNAL ERROR: Trying to pop from empty stack!')
  else begin
    pop := Top^.Value;
    DelUnit := Top;
    Top := Top^.Next;
    if Top <> nil then Top^.Prev := nil;
    Dispose(DelUnit);
  end;
end;

function TParserStack.Peek: TStackValue;
begin
  if Empty then raise EStackEmpty.Create('INTERNAL ERROR: Trying to peek from empty stack!')
  else peek := Top^.Value;
end;

function TParserStack.Empty: boolean;
begin
  empty := Top=nil;
end;

end.

